import Ajax from '../../util/ajax'
import api from '../../util/api'

export default {
  namespaced: true,
  state: {
    types: {},
    recommends: {
      list: []
    },
    books: {},
    minebooks: {},
  },
  mutations: {
    updateTypeList (state, payload) {
      state.types = payload
    },
    updateRecommendBooks (state, payload) {
      state.recommends.list = state.recommends.list.concat(payload.list)
    },
    updateBookList (state, payload) {
      state.books = payload
    },
    updateMineBook (state, payload) {
      state.minebooks = payload
    },
  },
  actions: {
    getBookType ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        Ajax({
          ...api.library.getNavList,
          queryData: {
            ...payload
          },
          urlData: {
            caseid: payload.caseid,
          }
        }).then(res => {
          commit('updateTypeList', res)
          resolve(res)
        }).catch((err) => {
          reject(err)
        })
      })
    },
    getRecommendBook ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        Ajax({
          ...api.library.getRecommendBookList,
          queryData: {
            ...payload
          },
          urlData: {
            caseid: payload.caseid,
          }
        }).then(res => {
          commit('updateRecommendBooks', res)
          resolve(res)
        }).catch((err) => {
          reject(err)
        })
      })
    },
    getBookList ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        Ajax({
          ...api.library.getBookList,
          queryData: {
            ...payload
          },
        }).then(res => {
          commit('updateBookList', res)
          resolve(res)
        }).catch((err) => {
          reject(err)
        })
      })
    },
    getMineBook ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        Ajax({
          ...api.library.getMineRecord,
          queryData: {
            ...payload
          },
        }).then(res => {
          commit('updateMineBook', res)
          resolve(res)
        }).catch((err) => {
          reject(err)
        })
      })
    },
    reserveBook (c, { bookid }) {
      return new Promise((resolve, reject) => {
        Ajax({
          ...api.library.reserve,
          urlData: {
            bookid,
          },
        }).then(res => {
          resolve(res)
        }).catch((err) => {
          reject(err)
        })
      }) 
    }
  }
}